      SUBROUTINE MeanFlow (IBC, IQS, AMach, Re, Re0, Iprint, Tstag)
C************************************************************************
C*    COMPRESSIBLE BOUNDARY LAYER via SIMILARITY TRANS.
C*               FLAT-PLATE AT ZERO INCIDENCE
C*    
C*                   Fabio Bertolotti,  May 1989.
C*                 Modified Aug 89 for variable Cp
C*
C*   ALL VARIABLES NONDIMENSIONALIZED WITH THE FREE-STREAM VALUES.
C*   SOLUTION VIA SHOOTING METHOD EMPLOYING A FOURTH ORDER RUNGE-KUTTA
C*   IN CONJUNCTION WITH A NEWTON PROCEDURE.
C*
C*   a) VARIABLE Cp
C*   b) Cp, Mu, and K GIVEN BY FOURTH ORDER POLYNOMIAL IN Temp
C*             Cp = Cp0 + T*(Cp1 + T*(Cp2 + T*(Cp3 + T*Cp4)))
C*             Mu = Vi0 + T*(Vi1 + T*(Vi2 + T*(Vi3 + T*Vi4)))
C*             K  = Cn0 + T*(Cn1 + T*(Cn2 + T*(Cn3 + T*Cn4)))
C*      IN THIS FORMULA ALL QUANTITIES ARE DIMENSIONAL. THE REFRENCE
C*      VALUES FOR Cp, Mu & K ARE CpRef, ViRef, CnRef.
C*
C*   c) EQUATION OF STATE:   Rho       =  1/T   (Pressure constant in y)
C*   d) REFERENCE LENGTH :   Delta(x0) = sqrt( Mu*X0/U*Rho )
C*   e) SIMILARITY VARIAB:   eta       = YY/Delta(x)
C*   f) DORODNITSYN TRANS:   YY        = Integral (rho dy)
C*
C*   INPUT VARIABLE:
C*      IBC = 0  TEMPERATURE AT WALL SPECIFIED
C*      IBC = 1  TEMPERATURE GRADIENT AT WALL SPECIFIED
C*      IQS = 0  QUESS THE INITIAL CONDITIONS FOR THE RUNGE-KUTTA
C*      IQS = 1  USE PREVIOUSLY CALC. INITIAL CONDITIONS
C*
C*   VARIABLES:
C*      AMACH = MACH NUMBER
C*      Re  = CURRENT REYNOLDS NUMBER    (Used in V component calc)
C*      Gam = RATIO OF SPECIFIC HEATS
C*      Pr  = FREE STREAM PRANDL NUMBER  (Cp Mu / k)
C*      Ec  = FREE STREAM ECKER NUMBER   (U^2 / (Cp T))
C*      Tstag  = IMENSIONAL STAGNATION TEMPERATURE (K)
C*      Twall  = NONDIMENSIONAL WALL TEMPERATURE
C*      TPwall = NONDIMENSIONAL TEMPERATURE GRADIENT AT WALL
C*      Y   = NONDIMENSIONAL PLATE NORMAL PHYSICAL COORDINATE
C*      X   = NONDIMENSIONAL STREAMWISE PHYSICAL COORDINATE
C*      YY  = DORODNITSYN TRANSFORMED Y,  YY = Int ( rho * dy )
C*      Eta = SIMILARITY VARIABLE, Eta = YY/Delta
C*      Delta = sqrt( 2*Mu*U**2*x/Rho )     U,rho,Mu AT FREE STREAM.
C*      F   = NONDIMENSIONAL STREAM FUNC, F(eta)
C*      T   = NONDIMENSIONAL TEMPERATURE, T(eta) 
C*      Rho = DENSITY
C*      Um  = NOND. U COMPONENT OF VELOCITY, AND DERIV. ' & '' AT COLL.
C*      Vm  = NOND. V COMPONENT OF VELOCITY, AND DERIV. ' & '' AT COLL.
C*      Tm  = NOND. TEMPERATURE            , AND DERIV. ' & '' AT COLL.
C*      G   = Partial YY / Partial X. (USED IN V VELOCITY EVALUATION)
C*
C*      RUNGE-KUTTA SET-UP: 
C*            Z(1) = F''       Z(2) = F'      Z(3) = F
C*                             Z(4) = T'      Z(5) = T
C*                                            Z(6) = Y
C*                                            Z(7) = G
C*                                            Z(8) = Eta
C*          
C*        DZ(1) = x-mometum equation
C*        DZ(2) = Z(1)
C*        DZ(3) = Z(2)
C*        DZ(4) = energy equation
C*        DZ(5) = Z(4)
C*        DZ(6) = Inverse Dorodnitsyn trans, dY = T*(dEta)
C*        DZ(7) = partial YY / partial x equation
C*
C* OUTPUT:  ARRAY Zs(602,8)  holding vatriables Z(1) through Z(8)
C*                            at every-other step in eta.
C*
C************************************************************************
#include "common.ins"
      DIMENSION      Z(7),DZ(7),Q(7),Zs(602,8),BZ(7),Er1(3),Er2(3)

C MAXIMUM ERROR IN SHOOTING, NUMBER OF STEPS
      ErrorMax = 1.0e-10
      Nsteps   = 1200  
      Nsave    = Nsteps/2

C BOUNDARY CONDITIONS: QUESS IF SO WISHED.
      BZ(1) = UPwall
      BZ(2) = 0
      BZ(3) = 0
      BZ(4) = TPwall
      BZ(5) = Twall
      BZ(6)  = 0
      BZ(7)  = 0
      IF (IQS . EQ. 0 ) THEN
         CALL QUESS(IBC, BZ1, BZ4, BZ5, AMach, Re, Re0)
         BZ(1) = BZ1
         BZ(4) = BZ4
         BZ(5) = BZ5
      END IF

      if (Iprint.gt.0) then 
      write (6,*)
      write (6,*) ' Guess: fpp(0) = ', BZ(1)
      write (6,*) ' Guess:  tp(0) = ', BZ(4)
      write (6,*) ' Guess:   t(0) = ', BZ(5)
      write (6,*) ' Guess: BLedge = ', BLedge
      write (6,*)
      end if

C INITIAL POINT OF NEWTON ITERATION
      Iter  = 0
30    Iter  = Iter + 1
      Itrip = 0

C SET BOUNDARY CONDITIONS
20    DO 21 Ju = 1, 7
21      Z(Ju) = BZ(Ju)
 
C SAVE DATA AT WALL: ALSO INITIAL POINT OF NEWTON ITERATION
      IF (Itrip.EQ.0) THEN
         J1 = 1
         DO 18 Ju = 1, 7
18         Zs(J1,Ju) = Z(Ju)
         Zs(J1,8) = 0
      END IF

C STEP FROM WALL TO BLedge WITH VARIABLE STEP SIZE, 3*NSTEPS/4 STEPS
C STEP FROM BLedge TO 2*BLedge WITH FIXED STEP SIZE, NSTEPS/4 STEPS
      ID   = 0
      IS   = NSTEPS
      Et   = 0
      Nedg = 3*NSTEPS/4
      H0   = 3*BLedge/(2*Nedg**(1.5))
      H1   = BLedge/(NSTEPS/4)
      H    = H0
 
C RUNGE-KUTTA BY GILLS METHOD
12    ID=ID+1
      GO TO (1,4,5,3,7),ID
1     DO 2 I=1,7
2     Q(I)=0
      A=.5d0
      GO TO 9
3     A=1.707106781186548d0
4     Et =Et + H/2.0d0
5     DO 6 I=1,7
      Z(I)=Z(I)+A*(DZ(I)*H-Q(I))
6     Q(I)=2*A*H*DZ(I)+(1.0d0-3*A)*Q(I)
      A=.292893218813452d0

C Cp, Mu, and K via POLYNOMIAL IN TEMPERATURE:                     
9     Rho = 1/Z(5)
      T0  = Z(5)
      T1  = Z(4)
      call Thermo (T0, Visc, Viscp, Cond, Condp, Cp, Prand)
      C10 = Rho*Visc
      C20 = Rho*Cond
      C30 = Cp
      C11 = -T1*C10/T0 + Rho*Viscp*T1  
      C21 = -T1*C20/T0 + Rho*Condp*T1  

C X-MOMEMTUM EQUATION
      DZ(1) = (-Z(3)*Z(1)/2 - Z(1)*C11)/C10
      DZ(2) = Z(1)
      DZ(3) = Z(2)

C ENERGY EQUATION
      DZ(4) = (-Z(4)*(C21/Pr + C30*Z(3)/2) - C10*Ec*Z(1)**2)*Pr/C20
      DZ(5) = Z(4)

C DORODNITSYN INVERSE TRASFORM:  dy = T dYY
      DZ(6) = Z(5)

C EQUATION FOR partial YY / partial X  (needed to calc V velocity)
      DZ(7) = (Z(4)/Z(5))*(Et/(2*Re) - Z(7))

      GO TO 12
7     DO 8 I=1,7
8     Z(I)=Z(I)+H*DZ(I)/6-Q(I)/3
      ID=0
      IS=IS-1

C PRESENT ETA
      Z(8) = Et 
c     write (6,*) ' Et, T ', Et, Z(5)

C VARIABLE STEP SIZE
      H = H0*sqrt(FLOAT((NSTEPS-IS)))
      IF (IS.LT.NSTEPS/4) H = H1

C SAVE DATA FOR LATER EVALUATION OF VELOCITY FIELD
      IF ((Itrip.eq.0).AND.(MOD(IS,2).EQ.0)) THEN
        J1 = J1 + 1
        DO 19 Ju = 1, 7
19        Zs(J1,Ju) = Z(Ju)
        Zs(J1,8) = Et
      END IF
       
C CONTINUE INTEGRATION
      IF (IS.GT.0) GO TO 12

C ****************************************************************
C *                 NEWTON ITERATION ON BC                       *
C *                                                              *
C * UDATE USES 3 RUNGE-KUTTA 'SHOTS'                             *
C * ONE SHOT IS FOR FINITE DIFF. EVALUATION OF                   *
C *          d(Er1)/d(DB1)   AND  d(Er2)/d(BD1)                  *
C * ONE SHOT IS FOR FINITE DIFF. EVALUATION OF                   *
C *          d(Er1)/d(DB2)   AND  d(Er2)/d(BD2)                  *
C *                                                              *
C * WHERE:                                                       *
C *  DB1 = MODIFICATION TO F' BOUNDARY CONDITION                 *
C *  DB2 = MODIFICATION TO T or T' BOUNDARY CONDITION            *
C *  Er1 = ERROR IN U VELOCITY OUTSIDE B.L.                      *
C *  Er2 = ERROR IN TEMPERATURE OUTSIDE B.L.                     *
C *                                                              *
C ****************************************************************

C DEFINE ERRORS
      Itrip = Itrip + 1
      Er1(Itrip) = 1.0d0 - Z(2)
      Er2(Itrip) = 1.0d0 - Z(5)

C INCREMENT BC. FOR FINITE DIFF EVALUATION OF ERROR GRADIENT
      GOTO (100, 101, 102) Itrip
100   DBZ1  = BZ(1)*0.01 
      IF (ABS(DBZ1).LT.1E-6) DBZ1 = 1.0E-06
      BZ(1) = BZ(1) + DBZ1
      GOTO 20
101   BZ(1) = BZ(1) - DBZ1
      IF (IBC.EQ.0) THEN
         DBZ2  = BZ(4)*0.10
         IF (ABS(DBZ2).LT.1E-6) DBZ2 = 1.0E-05
         BZ(4) = BZ(4) + DBZ2
      ELSE
         DBZ2  = BZ(5)*0.01
         IF (ABS(DBZ2).LT.1E-6) DBZ2 = 1.0E-06
         BZ(5) = BZ(5) + DBZ2
      END IF
      GOTO 20

102   CONTINUE
      IF (IBC.EQ.0) THEN
         BZ(4) = BZ(4) - DBZ2
      ELSE
         BZ(5) = BZ(5) - DBZ2
      END IF

C EVALUATE JACOBIAN VIA FINITE DIFFERENCE
      DJ11 = (Er1(2) - Er1(1))/DBZ1
      DJ12 = (Er1(3) - Er1(1))/DBZ2
      DJ21 = (Er2(2) - Er2(1))/DBZ1
      DJ22 = (Er2(3) - Er2(1))/DBZ2
      DET  =  DJ11*DJ22 - DJ21*DJ12
      DBZ1 = -(Er1(1)*DJ22 - Er2(1)*DJ12)/DET
      DBZ2 = -(DJ11*Er2(1) - DJ21*Er1(1))/DET
      BZ(1) = BZ(1) + DBZ1
      IF (IBC.EQ.0) THEN
         BZ(4) = BZ(4) + DBZ2
      ELSE
         BZ(5) = BZ(5) + DBZ2
      END IF
     
      if (Iprint.gt.0) then
      write (6,*) 'basic flow convergence = ', Er1(1), Er2(1)
      end if

C CHECK CONVERGENCE 
      IF (abs(Er1(1)) + abs(Er2(1)) .GT. ErrorMax) GOTO 30

C SAVE WALL VALUES
      UPwall = Zs(1,1)
      TPwall = Zs(1,4)
      Twall  = Zs(1,5)

C INTERPOLATE AT COLLOCATION POINTS.
      CALL INTERPOL (Zs, Nsave, AMach, Re, Re0)

C GASS CONSTANT FOR AIR, IN (J/Kg*K)
      Rair = 286.83

C EVALUATE FREE STREAM VELOCITY (m/s) 
      Ufree = AMach*sqrt(Gam*Rair*Te)
     
      write (6,*) ' Ufree (m/s) = ', Ufree

C EVALUATE FREE STREAM STAGNATION TEMPERATURE, VIA SUCCESSIVE APPROX.
      V0    = Cp0*Te + Cp1*Te**2/2 + Cp2*Te**3/3 + 
     &        Cp3*Te**4/4 + Cp4*Te**5/5
      DTs   = max(Te/20, 5.0)
      Tstag = Te
310   Tstag = Tstag + DTs
      V1    = Cp0*Tstag + Cp1*Tstag**2/2 + Cp2*Tstag**3/3 + 
     &        Cp3*Tstag**4/4 + Cp4*Tstag**5/5
      IF (DTs.lt.0.01) GOTO 320
      IF (V1-V0.lt.Ufree**2/2) GOTO 310
      Tstag = Tstag - DTs
      DTs   = DTs/4
      GOTO 310
320   CONTINUE
      
C FINISHED
      RETURN
      END

      SUBROUTINE QUESS(IBC, BZ1, BZ4, BZ5, AMach, Re, Re0)   
C **************************************************************
C * 
C *  QUESS BOUNDARY CONDITION: 
C *  BZ1 = F'' at wall 
C *  BZ4 = T'  at wall 
C *  BZ5 = T   at wall
C *
C *  IBC=0   Wall Temp Secified
C *  IBC=1   Insulated Wall
C *
C *  USE SOMMER-SHORT REFERNCE STATE. (NACA TN-3391, 1955)
C *
C **************************************************************
#include "common.ins"

C APPROXIMATE ADIABATIC WALL TEPERATURE
      Taw = 1.0 + SQRT(Pr)*Ec/2.0

C SET WALL TEMPERATURE
      IF (IBC.EQ.1) THEN
         Tw = Taw
      ELSE
         Tw = Twall
      END IF

C SUMMER-SHORT REFERENCE STATE: NONDIMENSIOANLIZED WITH Te
      Tref = 1 + 0.45*(Tw - 1) + 0.035*AMACH**2

C DENSITY, Cp, VISCOSITY & CONDUC (Polynomial), PRANDL No AT REFERENCE STATE
      Dens = 1/Tref
      call Thermo (Tref, Visc, Viscp, Cond, Condp, Cp, PrRef)

C DENSITY, VISCOSITY & CONDUC (Polynomial), AT WALL
      DensW = 1/Tw 
      call Thermo (Tw, ViscW, ViscWp, Condw, CondWp, CpW, PrW)

C ESTIMATE B.L EDGE
      DsW  = 1/Taw 
      call Thermo (Taw, VsW, dummy, dummy, dummy, dummy, dummy)
      BLedge = 7*sqrt(VsW/DsW)

C REYNOLDS ANALOGY FACTOR, s
      Sre  = (PrRef)**(2.0/3.0)

C ADIABATIC WALL TEPERATURE AT REFERENCE STATE
      Taw = 1.0 + SQRT(PrRef)*Ec/2.0

C SKIN FRICTION
      BZ1 = 0.3320573362236681d0*sqrt(Dens*Visc)/(DensW*ViscW)

C HEAT FLUX AT WALL
      TPw = 0.332057*sqrt(Dens*Visc)*Dens*(Taw - Tw)*Pr/Sre
      TPw = TPw/(DensW*CondW)

C SET TEMP BC
      IF (IBC.EQ.0) THEN
         BZ5 = Tw
         BZ4 = TPw
      ELSE
         BZ4 = TPwall
         BZ5 = Taw
      END IF

C FINISHED
      RETURN
      END

      
      SUBROUTINE INTERPOL (Zs, IZmax, AMach, Re, Re0)
C ********************************************************************
C *
C * INTERPOLATE TO GET VELOCITIES AND TEMPERATURE AT COLLOCATION  
C * POINTS USING CUBIC SPLINE INTERPOLATION.
C *
C * INPUT:   Zs(1000,8)  See BFLOW for description
C *          Re  = Current Reynolds number
C *          Re0 = Reynolds number at location of nondimensionalization
C *
C * OUTPUT:
C *          UM(j,0) = U velocity at point CC(j)
C *          UM(j,1) = dU/dy
C *          UM(j,2) = d^2U/ dy^2
C *          VM(j,0) = V velocity  (same as U for index 1,2)
C *          TM(j,0) = Temperature (same as U for index 1,2)
C *          UMx(j)  = dU/dx
C *          TMx(j)  = dT/dx
C *
C * BOUNDARY LAYER GROWTH FACTOR: Re0/Re
C *
C ********************************************************************
#include "common.ins"
      DIMENSION      Zs(602,8), CC(0:Jx)

C STORE COLLOCATION POINTS
      do 1 J=0,Jx
1       CC(j) = Eta(j)

C BOUNDARY LAYER GROWTH FACTOR
      bgf   = Re0/Re

C SEE HOW MANY COLL POINTS IN RUNGE-KUTTA INTEG. DOMAIN
      JX1 = 0
5     continue      
      if ((CC(JX1).lt.Zs(IZmax,6)/bgf).and.(JX1.lt.JX)) then
        JX1 = JX1 + 1
        goto 5
      end if
      JX1 = JX1 - 1

C INTERPOLATE FOR J < JX1
      in = 1
      do 20 j=0,JX1

C SEARCH FOR N, N+1 INTERVAL CONTAINING COLL PT CC(j)
10    continue 
      Y0  = Zs(in  ,6)/bgf
      Y1  = Zs(in+1,6)/bgf
      Yp0 = Zs(in  ,5)/bgf
      Yp1 = Zs(in+1,5)/bgf
      if ( (Y1.gt.CC(j)).and.(Y0.le.CC(j)) ) then 
      
C SET VALUES AT LOWER AND UPPER POINT N, N+1 
      Eta0 = Zs(in  ,8)
      Eta1 = Zs(in+1,8)
      F0   = Zs(in  ,3)
      F1   = Zs(in+1,3)
      Fp0  = Zs(in  ,2)
      Fp1  = Zs(in+1,2)
      Fpp0 = Zs(in  ,1)
      Fpp1 = Zs(in+1,1)
      T0   = Zs(in  ,5)
      T1   = Zs(in+1,5)
      Tp0  = Zs(in  ,4)
      Tp1  = Zs(in+1,4)
      G0   = Zs(in  ,7)
      G1   = Zs(in+1,7)
      Gp0  = (Tp0/T0)*(Eta0/(2*Re) - G0)
      Gp1  = (Tp1/T1)*(Eta1/(2*Re) - G1)

C THIRD DERIVATIVE OF F AND SECOND DERIVATIVE OF T AT N, N+1
      Rho = 1/T0  
      call Thermo (T0, Visc, Viscp, Cond, Condp, Cp, Prand)
      C10 = Rho*Visc
      C20 = Rho*Cond
      C30 =     Cp
      C11 = -Tp0*C10/T0 + Rho*Viscp*Tp0
      C21 = -Tp0*C20/T0 + Rho*Condp*Tp0
      Fppp0 = (-F0*Fpp0/2 - Fpp0*C11)/C10
      Tpp0  = (-Tp0*C21/Pr - C30*F0*Tp0/2 - C10*Ec*Fpp0**2)*Pr/C20

      Rho = 1/T1  
      call Thermo (T1, Visc, Viscp, Cond, Condp, Cp, Prand)
      C10 = Rho*Visc
      C20 = Rho*Cond 
      C30 =     Cp
      C11 = -Tp1*C10/T1 + Rho*Viscp*Tp1
      C21 = -Tp1*C20/T1 + Rho*Condp*Tp1
      Fppp1 = (-F1*Fpp1/2 - Fpp1*C11)/C10
      Tpp1  = (-Tp1*C21/Pr - C30*F1*Tp1/2 - C10*Ec*Fpp1**2)*Pr/C20

C SOLVE FOR Eta CORRESPONDING TO COLLOCATION PT. VIA NEWTON ITERATION
C OF SPLINE FUNCTION OVER INTERVAL.
      De  = Eta1 - Eta0
      De2 = De*De
      De3 = De*De*De
      AA1 =  3*(Y1 - Y0)/De2 - (2*Yp0 + Yp1)/De
      AA2 = -2*(Y1 - Y0)/De3 + (Yp0 + Yp1)/De2
      Dc  = (CC(j) - Y0)/Yp0
15    YE  = Y0 + Dc*(Yp0 + Dc*(AA1 + Dc*AA2)) - CC(j)
      YEp = Yp0 + 2*AA1*Dc + 3*AA2*Dc*Dc    
      DDc = -YE/YEp
      Dc  = Dc + DDc
      if (abs(DDc).gt.1d-10) goto 15
      Dc2 = Dc*Dc
      Dc3 = Dc*Dc*Dc
      EtaC= Eta0 + Dc

C INTERPOLATED VALUES: F, Fp, Fpp, T, Tp, G
      AA1  =  3*(F1 - F0)/De2 - (2*Fp0 + Fp1)/De
      AA2  = -2*(F1 - F0)/De3 + (Fp0 + Fp1)/De2
      FC   = F0 + Fp0*Dc + AA1*Dc2 + AA2*Dc3
      
      AA1  =  3*(Fp1 - Fp0)/De2 - (2*Fpp0 + Fpp1)/De
      AA2  = -2*(Fp1 - Fp0)/De3 + (Fpp0 + Fpp1)/De2
      FCp  = Fp0 + Fpp0*Dc + AA1*Dc2 + AA2*Dc3
      
      AA1  =  3*(Fpp1 - Fpp0)/De2 - (2*Fppp0 + Fppp1)/De
      AA2  = -2*(Fpp1 - Fpp0)/De3 + (Fppp0 + Fppp1)/De2
      FCpp = Fpp0 + Fppp0*Dc + AA1*Dc2 + AA2*Dc3
      
      AA1  =  3*(T1 - T0)/De2 - (2*Tp0 + Tp1)/De
      AA2  = -2*(T1 - T0)/De3 + (Tp0 + Tp1)/De2
      TC   = T0 + Tp0*Dc + AA1*Dc2 + AA2*Dc3
      
      AA1  =  3*(Tp1 - Tp0)/De2 - (2*Tpp0 + Tpp1)/De
      AA2  = -2*(Tp1 - Tp0)/De3 + (Tpp0 + Tpp1)/De2
      TCp  = Tp0 + Tpp0*Dc + AA1*Dc2 + AA2*Dc3

C THIRD DERIVATIVE OF F AND SECOND DERIVATIVE OF T AT CC(j)
      Rho = 1/Tc  
      call Thermo (Tc, Visc, Viscp, Cond, Condp, Cp, Prand)
      C10 = Rho*Visc
      C20 = Rho*Cond
      C30 = Cp
      C11 = -Tcp*C10/Tc + Rho*Viscp*Tcp
      C21 = -Tcp*C20/Tc + Rho*Condp*Tcp
      FCppp = (-FC*FCpp/2 - FCpp*C11)/C10
      TCpp  = (-TCp*C21/Pr - C30*FC*TCp/2 - C10*Ec*FCpp**2)*Pr/C20

C EVALUES OF dY/dx
      AA1  =  3*(G1 - G0)/De2 - (2*Gp0 + Gp1)/De
      AA2  = -2*(G1 - G0)/De3 + (Gp0 + Gp1)/De2
      GC   = G0 + Gp0*Dc + AA1*Dc2 + AA2*Dc3
      GCp  = (TCp/TC)*(EtaC/(2*Re) - GC)
      GCpp = (TCpp/TC - (TCp/TC)**2)*(EtaC/(2*Re) - GC) + 
     &        TCp*(1/(2*Re) - GCp)/TC

C U VELOCITY AT COLL. PT.
      UM(J,0) = FCp
      UM(J,1) = (FCpp/TC)*bgf
      UM(J,2) = (FCppp - FCpp*TCp/TC)*(bgf/TC)**2
      UMx(J)  = bgf*FCpp*(-EtaC/(2*Re) + GC)

C V VELOCITY AT COLL. PT.
      EtaR     = EtaC/(2*Re)
      VM(J,0) = TC*(-FCp*GC + (EtaC*FCp - FC)/(2*Re))
      VM(J,1) = -bgf*( (FCp*TCp/TC + FCpp)*(GC - EtaR) +
     &                   FC*TCp/(TC*2*Re) + FCp*GCp )
      VM(J,2) = bgf*bgf*(-1/TC)*( 
     &  (FCpp*TCp/TC + FCp*(TCpp/TC - (TCp/TC)**2) + FCppp)*(GC - EtaR)
     &  + (FCp*TCp/TC + 2*FCpp)*GCp + FCp*GCpp
     &  + ( FC*(TCpp/TC - (TCp/TC)**2) - FCpp )/(2*Re)        )

C TEMPERATURE AT COLL. PT.
      TM(J,0) = TC
      TM(J,1) = bgf*TCp/TC
      TM(J,2) = bgf*bgf*(TCpp/TC - (TCp/TC)**2)/TC
      TMx(J)  = bgf*TCp*(-EtaC/(2*Re) + GC)
      
C SAVE ETA VALUE
      Eta(J) = EtaC

C CONTINUE WITH LOOP OF COLL PT INSIDE RUNGE-KUTTA DOMAIN
      else
        in = in + 1
        goto 10
      end if
20    continue

C COLL. PTS OUTSIDE RUNGE-KUTTA DOMAIN
      EtaMx= Zs(IZmax,8) 
      F0   = Zs(IZmax,3)
      Fp0  = Zs(IZmax,2)
      Fpp0 = Zs(IZmax,1)
      T0   = Zs(IZmax,5)
      Tp0  = Zs(IZmax,4)
      G0   = Zs(IZmax,7)
      Disp = EtaMx - F0
      do 30 j = JX1+1, JX
         UM(j,0) = 1
         UM(j,1) = 0
         UM(j,2) = 0
         UMx(j)  = 0
         VM(j,0) = -G0 + Disp*bgf/(2*Re0)
         VM(j,1) = 0
         VM(j,2) = 0
         TM(j,0) = 1
         TM(j,1) = 0
         TM(j,2) = 0
30       TMx(j)  = 0

C EVALUATE DISPLACEMENT THICKNESS (TRAPZOI): STORE IN LAST ELEMENT OF ETA
      displ = 0
      do 476 j=2, IZmax     
         ddy = Zs(j,6) - Zs(j-1,6)
         ur1 = Zs(j,2)/Zs(j,5)
         ur2 = Zs(j-1,2)/Zs(j-1,5)
         ur  = (ur1 + ur2)/2
476      displ = displ + (1 - ur)*ddy
      eta(jx)=displ

C FINISHED
      RETURN
      END


      SUBROUTINE Thermo (T, Visc, Viscp, Cond, Condp, Cp, Prand)
C *********************************************************************
C                      POLYNOMIAL APPROXIMATION
C T     = Nondimensional Temperature
C Visc  = Nondimensional Viscosity  
C Viscp = Nondimensional derivative Viscosity wrt Nondim Temperature
C Cond  = Nondimensional Conductivity
C Condp = Nondimensional derivative Conductivity wrt Nondim Temp.
C Cp    = Nondimensional specific heat
C Prand = Prandtl number
C *********************************************************************
#include "common.ins"

C REFERENCE QUANTITIES
      ViRef = Vi0+Te*(Vi1+Te*(Vi2+Te*(Vi3 + Te*Vi4)))
      CnRef = Cn0+Te*(Cn1+Te*(Cn2+Te*(Cn3 + Te*Cn4)))
      CpRef = Cp0+Te*(Cp1+Te*(Cp2+Te*(Cp3 + Te*Cp4)))
      Rair  = 286.83
      Gam   = 1.0/( 1.0 - Rair/CpRef) 
      Pr    = CpRef*ViRef/CnRef

C VALUES AT GIVEN TEMPERATURE
      Tz    =  T*Te
      Visc  = (Vi0+Tz*(Vi1+Tz*(Vi2+Tz*(Vi3 + Tz*Vi4))))/ViRef
      Cond  = (Cn0+Tz*(Cn1+Tz*(Cn2+Tz*(Cn3 + Tz*Cn4))))/CnRef
      Cp    = (Cp0+Tz*(Cp1+Tz*(Cp2+Tz*(Cp3 + Tz*Cp4))))/CpRef
      Viscp = Te*(Vi1+Tz*(2*Vi2+Tz*(3*Vi3 + Tz*4*Vi4)))/ViRef
      Condp = Te*(Cn1+Tz*(2*Cn2+Tz*(3*Cn3 + Tz*4*Cn4)))/CnRef
      Prand = Pr*Cp*Visc/Cond
      return
      end

      SUBROUTINE Thermz (T, Visc, Viscp, Cond, Condp, Cp, Prand)
C *********************************************************************
C                      SOUTHERLAND's LAW          
C T     = Nondimensional Temperature
C Visc  = Nondimensional Viscosity  
C Viscp = Nondimensional derivative Viscosity wrt Nondim Temperature
C Cond  = Nondimensional Conductivity
C Condp = Nondimensional derivative Conductivity wrt Nondim Temp.
C Cp    = Nondimensional specific heat
C Prand = Prandtl number
C *********************************************************************
#include "common.ins"

C REFERENCE QUANTITIES
      Pr    = 0.7
      Gam   = 1.4
      ViRef = 17.19E-06*(Te/273.1)**1.5*(273.1 + 110.5)/(Te + 110.5)
      CnRef = 2.373E-02*(Te/273.1)**1.5*(273.1 + 194.4)/(Te + 194.4)
      CpRef = Pr*CnRef/ViRef

C VALUES AT GIVEN TEMPERATURE
      Cp    = 1
      AS    = 110.5/Te 
      Visc  = T**1.5*(1+AS)/(T+AS)
      Cond  = Visc
      Viscp = T**0.5*(1+AS)*(1.5 - T/(T+AS))/(T+AS)
      Condp = Viscp
      Prand = Pr
      return
      end
